"""
@STCGoal Wrap what MLF does as Services

- Create MLF
- Read MLF (and create if doesn't exist and if we have a corresponding TTF PatternsData in the DB, we can also create the TTF)
- Update MLF
- Create TTF Patterns (data:  PatternName, PatternsColumns) 

Dependencies:
- MLFRequest
- TTF

Target Usage:
- mlfcli
- mlfapi
"""


import MLFRequest
import realityhelper


class MLFService:
    def __init__(self):
        pass
    def generate_features(self, request: MLFRequest):
        try:
            return realityhelper.generate_mlf_feature_pattern(
                request.instrument,
                request.timeframe,
                use_full=request.use_full,
                force_refresh=request.force_refresh,
                lag_period=request.lag_period,
                total_lagging_periods=request.total_lagging_periods,
                dropna=request.dropna,
                columns_to_keep=request.columns_to_keep,
                columns_to_drop=request.columns_to_drop,
                drop_bid_ask=request.drop_bid_ask,
                pn=request.patternname
            )
        except Exception as e:
            print(f"Error in generate_mlf_feature_pattern: {e}")
            print(f"patternname: {request.patternname} might just not have its prerequisite TTF/Pattern data. We would be running: jgtmlttfcli -i {request.instrument} -t {request.timeframe} -new")
    
    def create_request(self,instrument,timeframe,pn,lag_period=1,total_lagging_periods=5,use_full=True,force_refresh=False,dropna=True,columns_to_keep=None,columns_to_drop=None,drop_bid_ask=True):
        return MLFRequest(instrument,timeframe,pn,lag_period,total_lagging_periods,use_full,force_refresh,dropna,columns_to_keep,columns_to_drop,drop_bid_ask)


import dash
from dash import dcc, html
from dash.dependencies import Input, Output
import numpy as np
import pandas as pd

import plotly.graph_objects as go
from plotly.subplots import make_subplots
import plotly.express as px
import mplfinance as mpf

from datetime import datetime
import time
import jgtpy as jgt
from jgtpy import JGTCDS as cds
from jgtpy import JGTADS as ads
from jgtpy import adshelper as ah

from jgtpy import JGTPDSP as pds
from jgtpy.JGTChartConfig import JGTChartConfig
from jgtutils.jgtconstants import AO, AC, OPEN, HIGH, LOW, CLOSE, JAW, TEETH, LIPS


import os


main_plot_panel_id = 1
ao_plot_panel_id = 2
ac_plot_panel_id = 3

jaw_color = "#0000FF"
teeth_color = "#00FF00"
lips_color = "#FF0000"


def _make_alligator_line_plots_v2(data):
    jaw_plot = go.Scatter(
        x=data.index,
        y=data[JAW].values,
        # yaxis=f"y{main_plot_panel_id}",
        line=dict(color=jaw_color),
        mode="lines",
        name="Jaw",
    )
    teeth_plot = go.Scatter(
        x=data.index,
        y=data[TEETH].values,
        # yaxis=f"y{main_plot_panel_id}",
        line=dict(color=teeth_color),
        mode="lines",
        name="Teeth",
    )
    lips_plot = go.Scatter(
        x=data.index,
        y=data[LIPS].values,
        # yaxis=f"y{main_plot_panel_id}",
        line=dict(color=lips_color),
        mode="lines",
        name="Lips",
    )
    return jaw_plot, teeth_plot, lips_plot


def make_plot__ao_v5(data: pd.DataFrame, up_color="green", dn_color="red"):

    colors_ao = data["ao"].diff().apply(lambda x: up_color if x > 0 else dn_color)

    # Make 'ao' oscillator plot
    ao_plot = go.Bar(
        x=data.index,
        y=data["ao"].values,
        # yaxis=f"y{ao_plot_panel_id}",
        marker_color=colors_ao,
        name="AO",
    )

    return ao_plot


def make_plot__ac_v5(data: pd.DataFrame, up_color="darkgreen", dn_color="darkred"):

    colors_ac = data["ac"].diff().apply(lambda x: up_color if x > 0 else dn_color)

    # Make 'ac' oscillator plot
    ac_plot = go.Bar(
        x=data.index,
        y=data["ac"].values,
        # yaxis=f"y{ac_plot_panel_id}",
        marker_color=colors_ac,
        name="AC",
    )

    return ac_plot


def make_fig_v5():
    fig = make_subplots(
        rows=3,
        cols=3,
        column_widths=[0.13, 0.82, 0.05],
        row_heights=[0.6, 0.2, 0.2],
        row_titles=["Main", "AO", "AC"],
        shared_xaxes=True,
    )
    # fig = make_subplots(rows=2, cols=1, row_heights=[0.6, 0.4])
    return fig


def make_ohlc(data):
    ohlc_plot = go.Ohlc(
        x=data.index,
        open=data[OPEN],
        high=data[HIGH],
        low=data[LOW],
        close=data[CLOSE],
        # yaxis=f"y{main_plot_panel_id}",
    )
    return ohlc_plot


# timeframe = "D1"
# #i = "EUR/USD"
# instrument = "SPX500"

# no_cache=False

# data= ah.prepare_cds_for_ads_data(instrument=instrument,timeframe=timeframe)

# if no_cache or not os.path.exists("data.csv"):
#   data= ah.prepare_cds_for_ads_data(instrument=instrument,timeframe=timeframe)
#   data.to_csv("data.csv")
# else:
#   data=pd.read_csv("data.csv")


# Create Dash application
app = dash.Dash(__name__)  # Add the Flask application name

drop_down_instrument = dcc.Dropdown(
    id="instrument",
    options=[
        {"label": i, "value": i} for i in ["SPX500", "EUR/USD", "USD/JPY", "XAU/USD"]
    ],
    value="SPX500",
)
drop_down_timeframe = dcc.Dropdown(
    id="timeframe",
    options=[{"label": t, "value": t} for t in ["D1", "H4", "H1", "m15"]],
    value="D1",
)

# Define the app layout
chart_title = html.P(id="chart_title_p", children="JGT Interactive Learn Dash")
app.layout = html.Div(
    [
        dcc.Interval(
            id="interval-component",
            interval=1 * 60 * 1000,  # in milliseconds
            n_intervals=0,
        ),
        html.Table(
            [
                html.Tr(
                    id="",
                    className="",
                    children=[
                        html.Td(id="chart_title", className="", children=chart_title),
                        html.Td(id="", className="", children=drop_down_instrument),
                        html.Td(id="", className="", children=drop_down_timeframe),
                    ],
                ),
            ]
        ),
        dcc.Graph(id="live-graph", animate=True),
    ]
)


# Define callback to update graph
@app.callback(
    Output("live-graph", "figure"),
    [
        Input("interval-component", "n_intervals"),
        dash.dependencies.Input("instrument", "value"),
        dash.dependencies.Input("timeframe", "value"),
    ],
)
def update_graph_live(n, instrument, timeframe):
    # mpf_plot,axis,data = jgt.plot(instrument,timeframe,show=False)
    data = ah.prepare_cds_for_ads_data(instrument=instrument, timeframe=timeframe)

    # Make OHLC bars plot
    last_dt = str(data.index[-1])
    chart_title = " ADS Chart  " + instrument + " " + timeframe + " " + last_dt

    # jaw_plot,teeth_plot,lips_plot = make_plot_alligator(data)
    jaw_plot, teeth_plot, lips_plot = _make_alligator_line_plots_v2(data)

    ao_plot = make_plot__ao_v5(data, up_color="green", dn_color="red")

    ac_plot = make_plot__ac_v5(data, up_color="darkgreen", dn_color="darkred")

    ohlc_plot = make_ohlc(data)

    figure = make_fig_v5()

    figure.add_trace(jaw_plot, row=main_plot_panel_id, col=2)
    figure.add_trace(teeth_plot, row=main_plot_panel_id, col=2)
    figure.add_trace(lips_plot, row=main_plot_panel_id, col=2)

    # fig.add_trace(ao_plot, row=ao_plot_panel_id, col=1, secondary_y=False)
    # fig.add_trace(ao_plot, row=ao_plot_panel_id, col=1)
    figure.add_trace(ao_plot, row=ao_plot_panel_id, col=2)
    figure.add_trace(ac_plot, row=ac_plot_panel_id, col=2)

    figure.add_trace(ohlc_plot, row=main_plot_panel_id, col=2)

    figure.update_layout(
        autosize=True,
        xaxis_title="Date",
        yaxis_title="Price",
        # title=chart_title,
        height=1120,
        # Add your desired options here
        # For example:
        margin=dict(l=50, r=50, t=50, b=50),
        plot_bgcolor="rgba(0,0,0,0)",
        # paper_bgcolor='rgba(0,0,0,0)',
        # font=dict(family='Courier New, monospace', size=18, color='#7f7f7f')
    )

    return figure


# Run the app
if __name__ == "__main__":
    app.run_server(debug=True, port=8050, host="0.0.0.0")




#@STCIssue Any Mouth Related Signals have a relationship with Selling or Buying.  Can we represent this in the features?  For that, I think we have to consider the Model is going to evaluate a Selling or Buying Signal, therefore, can we represent the Mouth Signals for either Selling or Buying?  If so, how?  
#@STCGoal Wrap the Feature preparation done in these sessions.  As result: we have a file for an input POV in the JGTPY_DATA_FULL/mlf/$i_$t.csv
##@STCGoal It contains columns feature ready for the model.

""" # TO ADJUST....
  'price_peak_above', 'price_peak_bellow', 'ao_peak_above',
       'ao_peak_bellow', 'mfi_sq', 'mfi_green', 'mfi_fade', 'mfi_fake',
       'mfi_sig', 'mfi_str', 'mfi_str_M1', 'zcol_M1', 'ao_M1', 'mfi_str_W1',
       'zcol_W1', 'ao_W1'
"""
##@STCGoal It can be a source for the making of MX Data.

from jgtml import mfihelper2 as mfihelper, mxconstants as mxc, mxhelper as mxhelper,realityhelper,zonehelper
from jgtutils.jgtconstants import JAW,TEETH,LIPS,HIGH,LOW,BJAW,BTEETH,BLIPS,TJAW,TTEETH,TLIPS

#@STCIssue Requires validation from the jgtml_obsds_240515_TIDE_SIGNALS.py prototype
_NOT_VALIDATED__SEE_RAISED_MESSAGE = 'Not Validated, see: jgtml_obsds_240515_TIDE_SIGNALS.py for logics'



def __detect_directions(df,detected_direction_colname = 'detected_dir',colvalue_if_direction_is_sell = -1,colvalue_if_direction_is_buy = 1,colvalue_if_oscillating = 0):
  """
  Detect the directions of the normal alligator (JAW,TEETH,LIPS) with the relationship of current price (HIGH,LOW).
  
  Parameters:
  df: pd.DataFrame - the dataframe to detect the directions of the normal alligator using the 3 alligators existing columns (JAW,TEETH,LIPS) and the current price (HIGH,LOW)
  detected_direction_colname: str - the column name to store the detected direction
  colvalue_if_direction_is_sell: str - the column value if the direction is sell
  colvalue_if_direction_is_buy: str - the column value if the direction is buy
  colvalue_if_oscillating: str - the column value if the direction is oscillating
  
  Returns:
  pd.DataFrame - the dataframe with the directions of the normal alligator detected
  """
  
  #@STCGoal What is the optimal way to dectect direction ? 
  def evaluate_direction(row):
    if row[JAW] > row[TEETH] and row[TEETH] > row[LIPS] and row[HIGH] < row[LIPS]:
      return colvalue_if_direction_is_sell
    elif row[JAW] < row[TEETH] and row[TEETH] < row[LIPS] and row[LOW] > row[LIPS]:
      return colvalue_if_direction_is_buy
    else:
      return colvalue_if_oscillating

  
  #df[detected_direction_colname]= colvalue_if_direction_is_sell if df[JAW] > df[TEETH] and df[TEETH] > df[LIPS] and df[HIGH] < df[LIPS] else colvalue_if_direction_is_buy if df[JAW] < df[TEETH] and df[TEETH] < df[LIPS] and df[LOW] > df[LIPS] else colvalue_if_oscillating
  df[detected_direction_colname] = df.apply(evaluate_direction, axis=1)
  return df

from mlconstants import NORMAL_MOUTH_IS_OPEN_COLNAME
def __add_normal_mouth_is_open(df):
  """
  Add the Normal Mouth Is Open Flag to the dataframe.
  
  Parameters:
  df: pd.DataFrame - the dataframe to add the Normal Mouth Is Open Flag to
  
  Returns:
  pd.DataFrame - the dataframe with the Normal Mouth Is Open Flag added
  """
  detected_direction=__detect_directions(df)
  df[NORMAL_MOUTH_IS_OPEN_COLNAME] = (df[JAW] > df[TEETH]) & (df[TEETH] > df[LIPS])#GENERATED
  raise NotImplementedError(_NOT_VALIDATED__SEE_RAISED_MESSAGE)
  return df

from mlconstants import CURRENT_BAR_IS_OUT_OF_NORMAL_MOUTH_COLNAME
def __add_current_bar_is_out_of_normal_mouth(df):
  """
  Add the Current Bar Is Out of Normal Mouth Flag to the dataframe.
  
  Parameters:
  df: pd.DataFrame - the dataframe to add the Current Bar Is Out of Normal Mouth Flag to
  
  Returns:
  pd.DataFrame - the dataframe with the Current Bar Is Out of Normal Mouth Flag added
  """

  df[CURRENT_BAR_IS_OUT_OF_NORMAL_MOUTH_COLNAME] = (df[HIGH] > df[JAW]) | (df[LOW] < df[LIPS])#GENERATED
  raise NotImplementedError(_NOT_VALIDATED__SEE_RAISED_MESSAGE)
  return df


def _add_normal_alligator_flag_signals(df,
                    add_normal_mouth_is_open=True,
                    add_current_bar_is_out_of_normal_mouth=True
                    ):
  """
  Add the Normal Alligator Flag Signals to the dataframe.
  
  Parameters:
  df: pd.DataFrame - the dataframe to add the Normal Alligator Flag Signals to
  add_normal_mouth_is_open: bool - Add flag to output that tells if the normal alligator mouth is Open
  add_current_bar_is_out_of_normal_mouth: bool - Signal Out of the Normal Alligator Mouth
  
  Returns:
  pd.DataFrame - the dataframe with the Normal Alligator Flag Signals added
  """
  if add_normal_mouth_is_open:
    __add_normal_mouth_is_open(df)
  if add_current_bar_is_out_of_normal_mouth:
    __add_current_bar_is_out_of_normal_mouth(df)
  

from mlconstants import CURRENT_BAR_IS_IN_BIG_TEETH_COLNAME
def __add_current_bar_is_in_big_teeth(df):
  """
  Add the Current Bar Is In Big Teeth Flag to the dataframe.
  
  Parameters:
  df: pd.DataFrame - the dataframe to add the Current Bar Is In Big Teeth Flag to
  
  Returns:
  pd.DataFrame - the dataframe with the Current Bar Is In Big Teeth Flag added
  """
  df[CURRENT_BAR_IS_IN_BIG_TEETH_COLNAME] = (df[BTEETH] > df[BJAW]) & (df[BTEETH] > df[BLIPS])#GENERATED
  raise NotImplementedError(_NOT_VALIDATED__SEE_RAISED_MESSAGE)
  return df

#__add_big_mouth_is_open_and_current_bar_is_in_big_lips(df)
from mlconstants import BIG_MOUTH_IS_OPEN_AND_CURRENT_BAR_IS_IN_BIG_LIPS_COLNAME
def __add_big_mouth_is_open_and_current_bar_is_in_big_lips(df):
  """
  Add the Big Mouth Is Open and Current Bar Is In Big Lips Flag to the dataframe.
  
  Parameters:
  df: pd.DataFrame - the dataframe to add the Big Mouth Is Open and Current Bar Is In Big Lips Flag to
  
  Returns:
  pd.DataFrame - the dataframe with the Big Mouth Is Open and Current Bar Is In Big Lips Flag added
  """
  df[BIG_MOUTH_IS_OPEN_AND_CURRENT_BAR_IS_IN_BIG_LIPS_COLNAME] = (df[BJAW] > df[BTEETH]) & (df[BJAW] > df[BLIPS])#GENERATED
  raise NotImplementedError(_NOT_VALIDATED__SEE_RAISED_MESSAGE)
  return df

#__add_mouth_is_open_and_current_bar_is_in_big_teeth(df)
from mlconstants import MOUTH_IS_OPEN_AND_CURRENT_BAR_IS_IN_BIG_TEETH_COLNAME
def __add_mouth_is_open_and_current_bar_is_in_big_teeth(df):
  """
  Add the Mouth Is Open and Current Bar Is In Big Teeth Flag to the dataframe.
  
  Parameters:
  df: pd.DataFrame - the dataframe to add the Mouth Is Open and Current Bar Is In Big Teeth Flag to
  
  Returns:
  pd.DataFrame - the dataframe with the Mouth Is Open and Current Bar Is In Big Teeth Flag added
  """
  df[MOUTH_IS_OPEN_AND_CURRENT_BAR_IS_IN_BIG_TEETH_COLNAME] = (df[BLIPS] > df[BTEETH]) & (df[BLIPS] > df[BJAW]) #GENERATED
  raise NotImplementedError(_NOT_VALIDATED__SEE_RAISED_MESSAGE)
  return df


def _add_big_mouth_flag_signals(df,
                    add_current_bar_is_in_big_teeth=True,
                    add_big_mouth_is_open_and_current_bar_is_in_big_lips=True,
                    add_mouth_is_open_and_current_bar_is_in_big_teeth=True):
  """
  Add the Big Mouth Flag Signals to the dataframe.
  
  Parameters:
  df: pd.DataFrame - the dataframe to add the Big Mouth Flag Signals to
  add_current_bar_is_in_big_teeth: bool - Signal current bar is in Big Teeth
  add_big_mouth_is_open_and_current_bar_is_in_big_lips: bool - Signal Big Mouth is Open and Current Bar is in Big Lips
  add_mouth_is_open_and_current_bar_is_in_big_teeth: bool - Signal Big Mouth is Open and Current Bar is in Big Teeth
  
  
  Returns:
  pd.DataFrame - the dataframe with the Big Mouth Flag Signals added
  """
  if add_current_bar_is_in_big_teeth:
    __add_current_bar_is_in_big_teeth(df)
  if add_big_mouth_is_open_and_current_bar_is_in_big_lips:
    __add_big_mouth_is_open_and_current_bar_is_in_big_lips(df)
  if add_mouth_is_open_and_current_bar_is_in_big_teeth:
    __add_mouth_is_open_and_current_bar_is_in_big_teeth(df)













from mlconstants import CURRENT_BAR_IS_IN_TIDE_TEETH_COLNAME
def __add_current_bar_is_in_tide_teeth(df):
  """
  Add the Current Bar Is In Big Teeth Flag to the dataframe.
  
  Parameters:
  df: pd.DataFrame - the dataframe to add the Current Bar Is In Big Teeth Flag to
  
  Returns:
  pd.DataFrame - the dataframe with the Current Bar Is In Big Teeth Flag added
  """
  df[CURRENT_BAR_IS_IN_TIDE_TEETH_COLNAME] = (df[TTEETH] > df[TJAW]) & (df[TTEETH] > df[TLIPS])#GENERATED
  raise NotImplementedError(_NOT_VALIDATED__SEE_RAISED_MESSAGE)
  return df

#__add_tide_mouth_is_open_and_current_bar_is_in_tide_lips(df)
from mlconstants import TIDE_MOUTH_IS_OPEN_AND_CURRENT_BAR_IS_IN_TIDE_LIPS_COLNAME
def __add_tide_mouth_is_open_and_current_bar_is_in_tide_lips(df):
  """
  Add the Big Mouth Is Open and Current Bar Is In Big Lips Flag to the dataframe.
  
  Parameters:
  df: pd.DataFrame - the dataframe to add the Big Mouth Is Open and Current Bar Is In Big Lips Flag to
  
  Returns:
  pd.DataFrame - the dataframe with the Big Mouth Is Open and Current Bar Is In Big Lips Flag added
  """
  df[TIDE_MOUTH_IS_OPEN_AND_CURRENT_BAR_IS_IN_TIDE_LIPS_COLNAME] = (df[TJAW] > df[TTEETH]) & (df[TJAW] > df[TLIPS])#GENERATED
  raise NotImplementedError(_NOT_VALIDATED__SEE_RAISED_MESSAGE)
  return df

#__add_mouth_is_open_and_current_bar_is_in_tide_teeth(df)
from mlconstants import MOUTH_IS_OPEN_AND_CURRENT_BAR_IS_IN_TIDE_TEETH_COLNAME
def __add_mouth_is_open_and_current_bar_is_in_tide_teeth(df):
  """
  Add the Mouth Is Open and Current Bar Is In Big Teeth Flag to the dataframe.
  
  Parameters:
  df: pd.DataFrame - the dataframe to add the Mouth Is Open and Current Bar Is In Big Teeth Flag to
  
  Returns:
  pd.DataFrame - the dataframe with the Mouth Is Open and Current Bar Is In Big Teeth Flag added
  """
  df[MOUTH_IS_OPEN_AND_CURRENT_BAR_IS_IN_TIDE_TEETH_COLNAME] = (df[TLIPS] > df[TTEETH]) & (df[TLIPS] > df[TJAW]) #GENERATED
  raise NotImplementedError(_NOT_VALIDATED__SEE_RAISED_MESSAGE)
  return df



def _add_tide_mouth_flag_signals(df,
                    add_current_bar_is_in_tide_teeth=True,
                    add_tide_mouth_is_open_and_current_bar_is_in_tide_lips=True,
                    add_mouth_is_open_and_current_bar_is_in_tide_teeth=True):
  """
  Add the tide Mouth Flag Signals to the dataframe.
  
  Parameters:
  df: pd.DataFrame - the dataframe to add the tide Mouth Flag Signals to
  add_current_bar_is_in_tide_teeth: bool - Signal current bar is in tide Teeth
  add_tide_mouth_is_open_and_current_bar_is_in_tide_lips: bool - Signal tide Mouth is Open and Current Bar is in tide Lips
  add_mouth_is_open_and_current_bar_is_in_tide_teeth: bool - Signal tide Mouth is Open and Current Bar is in tide Teeth
  
  
  Returns:
  pd.DataFrame - the dataframe with the tide Mouth Flag Signals added
  """
  if add_current_bar_is_in_tide_teeth:
    __add_current_bar_is_in_tide_teeth(df)
  if add_tide_mouth_is_open_and_current_bar_is_in_tide_lips:
    __add_tide_mouth_is_open_and_current_bar_is_in_tide_lips(df)
  if add_mouth_is_open_and_current_bar_is_in_tide_teeth:
    __add_mouth_is_open_and_current_bar_is_in_tide_teeth(df)












_PTO_2407_NOT_IMPLEMENTED__or_JUST_LINKED_TO_CODE__MESSAGE = 'Not implemented yet.  See: pto_240706_patterning_helper.ipynb,pto_240706_patterning_helper_C02.ipynb,pto_240706_patterning_helper_C03.ipynb,pto_240706_patterning_helper_C04.ipynb,pto_240706_patterning_helper_C04b.ipynb,pto_240706__validation_240709.ipynb'
def _add_mfis_lag_feature(df):
  """
  Add the MFI Lag Feature to the dataframe.
  
  Parameters:
  df: pd.DataFrame - the dataframe to add the MFI Lag Feature to
  
  Returns:
  pd.DataFrame - the dataframe with the MFI Lag Feature added
  """
  raise NotImplementedError(_PTO_2407_NOT_IMPLEMENTED__or_JUST_LINKED_TO_CODE__MESSAGE)

def _add_zone_lag_feature(df):
  """
  Add the Zone Lag Feature to the dataframe.
  
  Parameters:
  df: pd.DataFrame - the dataframe to add the Zone Lag Feature to
  
  Returns:
  pd.DataFrame - the dataframe with the Zone Lag Feature added
  """
  raise NotImplementedError(_PTO_2407_NOT_IMPLEMENTED__or_JUST_LINKED_TO_CODE__MESSAGE)

def _add_ao_above_bellow_peaks_lag_feature(df):
  """
  Add the AO Above Bellow Peaks Lag Feature to the dataframe.
  
  Parameters:
  df: pd.DataFrame - the dataframe to add the AO Above Bellow Peaks Lag Feature to
  
  Returns:
  pd.DataFrame - the dataframe with the AO Above Bellow Peaks Lag Feature added
  """
  raise NotImplementedError(_PTO_2407_NOT_IMPLEMENTED__or_JUST_LINKED_TO_CODE__MESSAGE)

def _add_ao_lag_feature(df):
  """
  Add the AO Lag Feature to the dataframe.
  
  Parameters:
  df: pd.DataFrame - the dataframe to add the AO Lag Feature to
  
  Returns:
  pd.DataFrame - the dataframe with the AO Lag Feature added
  """
  raise NotImplementedError(_PTO_2407_NOT_IMPLEMENTED__or_JUST_LINKED_TO_CODE__MESSAGE)


def _get_data_for_instrument_timeframe(i,t,
                                       use_full=True,force_refresh=False,):
  """
  Get the data for the instrument and timeframe.
  
  Parameters:
  i: str - the instrument
  t: str - the timeframe
  use_full: bool - use the full data or not
  force_refresh: bool - force a refresh of the data or not
  
  Returns:
  pd.DataFrame - the data for the instrument and timeframe
  """
  from realityhelper import _load_ttf_data
  df=_load_ttf_data(i, t, use_full)
  raise NotImplementedError(_PTO_2407_NOT_IMPLEMENTED__or_JUST_LINKED_TO_CODE__MESSAGE)
  
def create_mlf_data__STUB(i,
                          t,
                          use_full=True,
                          force_refresh=False,
                          add_mfis_lag_feature=True,
                          add_zone_lag_feature=True,
                          add_ao_above_bellow_peaks_lag_feature=True,
                          add_ao_lag_feature=True,
                          add_normal_mouth_is_open=True,
                          add_current_bar_is_out_of_normal_mouth=True,
                          add_current_bar_is_in_big_teeth=True,
                          add_big_mouth_is_open_and_current_bar_is_in_big_lips=True,
                          add_mouth_is_open_and_current_bar_is_in_big_teeth=True,
                          add_current_bar_is_in_tide_teeth=True,
                          add_tide_mouth_is_open_and_current_bar_is_in_tide_lips=True,
                          add_mouth_is_open_and_current_bar_is_in_tide_teeth=True,
                    ):
  """
  Wrap all the creation we've been doing in the prototyping sessions. (2407) - See: pto_240706_patterning_helper.ipynb,pto_240706_patterning_helper_C02.ipynb,pto_240706_patterning_helper_C03.ipynb,pto_240706_patterning_helper_C04.ipynb,pto_240706_patterning_helper_C04b.ipynb,pto_240706__validation_240709.ipynb, $jgtml/samples/jgtml_obsds_240515_SIGNALS.result.csv
  
  Parameters:
  i: str - the instrument
  t: str - the timeframe
  use_full: bool - use the full data or not
  add_mfis_lag_feature: bool - add the mfi lag feature or not
  add_zone_lag_feature: bool - add the zone lag feature or not
  add_ao_above_bellow_peaks_lag_feature: bool - add the ao above bellow peaks lag feature or not
  add_ao_lag_feature: bool - add the ao lag feature or not (CURRENTLY IMPLEMENTED in the add_mfis_lag_feature)
  add_normal_mouth_is_open: bool - Add flag to output that tells if the normal alligator mouth is Open
  add_current_bar_is_out_of_normal_mouth: bool - Signal Out of the Normal Alligator Mouth
  add_current_bar_is_in_big_teeth: bool - Signal current bar is in Big Teeth
  add_big_mouth_is_open_and_current_bar_is_in_big_lips: bool - Signal Big Mouth is Open and Current Bar is in Big Lips
  add_mouth_is_open_and_current_bar_is_in_big_teeth: bool - Signal Big Mouth is Open and Current Bar is in Big Teeth
  add_current_bar_is_in_tide_teeth: bool - Signal current bar is in Tide Teeth
  add_tide_mouth_is_open_and_current_bar_is_in_tide_lips: bool - Signal Tide Mouth is Open and Current Bar is in Tide Lips
  add_mouth_is_open_and_current_bar_is_in_tide_teeth: bool - Signal Tide Mouth is Open and Current Bar is in Tide Teeth
  """
  df=_get_data_for_instrument_timeframe(i,t,use_full=use_full,force_refresh=force_refresh)
  
  if add_mfis_lag_feature:
    _add_mfis_lag_feature(df)
  if add_zone_lag_feature:
    _add_zone_lag_feature(df)
  if add_ao_above_bellow_peaks_lag_feature:
    _add_ao_above_bellow_peaks_lag_feature(df)
  if add_ao_lag_feature:
    _add_ao_lag_feature(df)
  
  
  _add_normal_alligator_flag_signals(df,add_normal_mouth_is_open=add_normal_mouth_is_open,
                    add_current_bar_is_out_of_normal_mouth=add_current_bar_is_out_of_normal_mouth)
    
  
  _add_big_mouth_flag_signals(df,
                    add_current_bar_is_in_big_teeth=add_current_bar_is_in_big_teeth,
                    add_big_mouth_is_open_and_current_bar_is_in_big_lips=add_big_mouth_is_open_and_current_bar_is_in_big_lips,
                    add_mouth_is_open_and_current_bar_is_in_big_teeth=add_mouth_is_open_and_current_bar_is_in_big_teeth)
  
  
  
  _add_tide_mouth_flag_signals(df,
                    add_current_bar_is_in_tide_teeth=add_current_bar_is_in_tide_teeth,
                    add_tide_mouth_is_open_and_current_bar_is_in_tide_lips=add_tide_mouth_is_open_and_current_bar_is_in_tide_lips,
                    add_mouth_is_open_and_current_bar_is_in_tide_teeth=add_mouth_is_open_and_current_bar_is_in_tide_teeth)
  raise NotImplementedError('Not implemented yet.  See: pto_240706_patterning_helper.ipynb,pto_240706_patterning_helper_C02.ipynb,pto_240706_patterning_helper_C03.ipynb,pto_240706_patterning_helper_C04.ipynb,pto_240706_patterning_helper_C04b.ipynb,pto_240706__validation_240709.ipynb')
  pass

  


import numpy as np
import os
import sys

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))
from jgtutils.jgtconstants import VOLUME,FDB_TARGET as TARGET
from mlconstants import ZONE_DEFAULT_COLNAME,MFI_DEFAULT_COLNAME

#from jgtpy import mfihelper,JGTCDSSvc as cdssvc
import anhelper as anh
import mxhelper
import mxconstants
import jtc
import pandas as pd


#@STCGoal We Have TTF Data with Lags for the pattern 'ttf_mfis_ao_2407a'
#@STCIssue How are created the TTF ?  How to create them with the lags and smaller Datasets (we dont need full)?

import ptottf 
#from jgtml import anhelper

from mfihelper2 import column_mfi_str_in_dataframe_to_id as _convert_mfi_columns_from_str_to_id

from zonehelper import wf_mk_zone_ready_dataset__240708 as _prep_zone_features_in_dataframe
#column_zone_str_in_dataframe_to_id as convert_zone_columns_from_str_to_id

from mxhelper import _add_lag_features_to_dataframe as _add_mfi_lagging_feature_to_ttfdf



def _load_ttf_data(i, t, use_full,force_refresh=False,quiet=True):
  if force_refresh:
    try:
      print("RH::loaddata::Upgrading/Refreshing the Depending Data before creating the TTF")
      ptottf._upgrade_ttf_depending_data(i, t, use_full=use_full, use_fresh=True,quiet=quiet)
      use_fresh=False
      #ptottf._upgrade_ttf_depending_data(i,t,use_full=use_full,use_fresh=True,quiet=quiet)
      ptottf.create_ttf_csv(i,t,use_full=use_full,use_fresh=use_fresh,quiet=quiet)
      force_refresh=False # We don't want to force refresh the next time
    except:
      print("ERROR::Failed to upgrade TTF depending data")
  return ptottf.read_ttf_csv(i, t, use_full=use_full,force_refresh=force_refresh)


#@STCIssue This should be Moved to mfihelper2.py
def __prep_mfi_2407a_features_in_dataframe(t, lag_period, total_lagging_periods, dropna, columns_to_keep, columns_to_drop, df):
    #df=_convert_mfi_columns_from_str_to_id(df,t, inplace=True)
  #add lags
    df=_add_mfi_lagging_feature_to_ttfdf(df,t,lag_period=lag_period, total_lagging_periods=total_lagging_periods,inplace=True)
    if dropna:
      df.dropna(inplace=True)
    if columns_to_keep:
      df=df[columns_to_keep]
    if columns_to_drop:
      for col in columns_to_drop:
        if col in df.columns:
          df.drop(columns=[col],inplace=True)
    return df


def _pto_get_dataset__with_mfi_ao__2407060929(i,t,lag_period=1, total_lagging_periods=5,dropna=True, use_full=True,columns_to_keep=None,columns_to_drop=None,force_refresh=False,quiet=True):
  #Read Data
  df=_load_ttf_data(i, t, use_full,force_refresh=force_refresh,quiet=quiet)
  #Convert the MFI columns from str to id before we add lags
  df = __prep_mfi_2407a_features_in_dataframe(t, lag_period, total_lagging_periods, dropna, columns_to_keep, columns_to_drop, df)
    #df.drop(columns=columns_to_drop,inplace=True)
  #columns_to_add_lags_to = mxhelper.get_mfi_features_column_list_by_timeframe(t)
  #ttfdf=anhelper.add_lagging_columns(ttfdf, columns_to_add_lags_to)
  return df

def get_mlf_basedir(use_full,ns="mlf"):
    if use_full:
        bd=os.getenv("JGTPY_DATA_FULL")
        if bd is None:
            raise Exception("JGTPY_DATA_FULL environment variable is not set.")
    else:
        bd=os.getenv("JGTPY_DATA")
        if bd is None:
            raise Exception("JGTPY_DATA environment variable is not set.")
    fulldir=os.path.join(bd,ns)
    #mkdir -p fulldir
    os.makedirs(fulldir, exist_ok=True)
    return fulldir

def get_mlf_outfile_fullpath(i,t,use_full,suffix="",ns="mlf"):
    save_basedir=get_mlf_basedir(use_full,ns)
    ifn=i.replace("/","-")
    output_filename = f"{ifn}_{t}_{suffix}.csv"
    return os.path.join(save_basedir,output_filename.replace("__","_"))
  
def create_pattern_dataset__ttf_mfis_ao_2407a_pto_get_dataset_we_need_in_here__2407060929(i,t,lag_period=1, total_lagging_periods=5,dropna=True, use_full=True,columns_to_keep=None,columns_to_drop=None,force_refresh=False,quiet=True):
  print("INFO::Requires experimentation with training, testing prediction to select from this what we need in reality to make the model work and predict reality of a signal.")
  df=_pto_get_dataset__with_mfi_ao__2407060929(i,t,lag_period=lag_period, total_lagging_periods=total_lagging_periods,dropna=dropna, use_full=use_full,columns_to_keep=columns_to_keep,columns_to_drop=columns_to_drop,force_refresh=force_refresh,quiet=quiet)
  output_filename=get_mlf_outfile_fullpath(i,t,use_full,"mfiao")
  try:
    df.to_csv(output_filename, index=True)
    print("INFO::MLF Saved to : ", output_filename)
  except:
    print("ERROR::Failed to save MLF to : ", output_filename)
  return df


def _read_adequate_pattern_dataset(i,t,use_full,pn,quiet=True,force_refresh=False):
  #@STCGoal Read ttf pattern
  from ptottf import read_ttf_csv
  df=read_ttf_csv(i, t, use_full=use_full,pn=pn,force_refresh=force_refresh)
  if not quiet:
    print(df.columns)
  return df
  #raise Exception("Not Implemented Yet::",patternname)

from mldatahelper import write_mlf_pattern_lagging_columns_list,read_patternname_columns_list
import mldatahelper as mldh
import anhelper

from mldatahelper import _get_lagging_columns_list

def generate_mlf_feature_pattern(i,t,lag_period=1, total_lagging_periods=5,dropna=True, use_full=True,columns_to_keep=None,columns_to_drop=None,drop_bid_ask=False,force_refresh=False,quiet=True,pn="ttf",out_lag_midfix_str='_lag_',just_keep_lagging_columns=False,save_to_csv=True,args=None):
  #@STCGoal Pattern Name -> We have the Columns list serialized
  columns_list_from_higher_tf = mldh.pndata__read_new_pattern_columns_list_with_htf(t,pn,args=args)
  if not quiet:
    print("INFO::Columns List from Higher TF to prep laggings using MLF:",columns_list_from_higher_tf) #@STCGoal We Know which columns were in the TTF Pattern when created.
  #print("-------------------------------------------------","Not Implemented Yet::",patternname)
  
  
  df:pd.DataFrame=_read_adequate_pattern_dataset(i,t,use_full,pn,force_refresh=force_refresh)
  
  df=__clean_dataframe(df, columns_to_keep, columns_to_drop, drop_bid_ask,dropna)
  
  df=anhelper.add_lagging_columns(df, columns_list_from_higher_tf, lag_period, total_lagging_periods, out_lag_midfix_str)
  
  lagging_columns=mldh._get_lagging_columns_list(columns_list_from_higher_tf, lag_period, total_lagging_periods, out_lag_midfix_str)
  
  fixed_columns_to_keep=columns_list_from_higher_tf+lagging_columns
  #write_patternname_columns_list
  if just_keep_lagging_columns:
    df=df[fixed_columns_to_keep]
  
  #print(df.columns)
  if save_to_csv:      
    write_mlf_pattern_lagging_columns_list(i, t, use_full, pn, fixed_columns_to_keep)
    #save the mlf df to_csv
    output_filename=get_mlf_outfile_fullpath(i,t,use_full,pn)
    df.to_csv(output_filename, index=True)
    print("INFO::MLF Saved to : ", output_filename) 

  
  return df



def get_mfis_ao_zone_2407b_feature(i,t,lag_period=1, total_lagging_periods=5,dropna=True, use_full=True,columns_to_keep=None,columns_to_drop=None,drop_bid_ask=False,force_refresh=False,quiet=True,zone_colname="",mfi_colname="",patternname="ttf"):
  print("WARN:::::: patternname is not IMPLEMENTED in MLF Now:",patternname)
  if zone_colname=="":
    zone_colname=ZONE_DEFAULT_COLNAME
  if mfi_colname=="":
    mfi_colname=MFI_DEFAULT_COLNAME
    
  df=_pto_get_dataset__with_mfi_ao__2407060929(i,t,lag_period=lag_period, total_lagging_periods=total_lagging_periods,dropna=dropna, use_full=use_full,columns_to_keep=columns_to_keep,columns_to_drop=columns_to_drop,force_refresh=force_refresh,quiet=quiet)
  
  df=_prep_zone_features_in_dataframe(df,t,lag_period=lag_period, total_lagging_periods=total_lagging_periods,inplace=True,zone_colname=zone_colname)
  
  __clean_dataframe(df, columns_to_keep, columns_to_drop, drop_bid_ask,dropna)
  
  output_filename=get_mlf_outfile_fullpath(i,t,use_full)
  try:
    df.to_csv(output_filename, index=True)
    print("INFO::MLF Saved to : ", output_filename)
  except:
    print("ERROR::Failed to save MLF to : ", output_filename)
  return df

from jgtutils.jgtconstants import BIDOPEN,BIDHIGH,BIDLOW,BIDCLOSE,ASKOPEN,ASKHIGH,ASKLOW,ASKCLOSE
def __clean_dataframe(df:pd.DataFrame, columns_to_keep=None, columns_to_drop=None, drop_bid_ask=False,dropna=True )->pd.DataFrame:
    """
    Common cleanup for the dataframe we are creating with various patterns
    
    """
    if dropna:
      df.dropna(inplace=True)
    if columns_to_keep:
      for col_name in columns_to_keep:
        if col_name not in df.columns:
          columns_to_keep.remove(col_name)
    if columns_to_drop:
      for col in columns_to_drop:
        if col in df.columns:
          df.drop(columns=[col],inplace=True)
    if drop_bid_ask:
      bid_ask_columns = [BIDOPEN,BIDHIGH,BIDLOW,BIDCLOSE,ASKOPEN,ASKHIGH,ASKLOW,ASKCLOSE]
      for col in bid_ask_columns:
        if col in df.columns:
          df.drop(columns=[col],inplace=True)
    return df
  